/**
@file
@date        04/09/2014
@author      Josef Hofer 
@copyright   CERN (EN-ICE-SCD)

@brief Library containing general functions used by the unLHCLoggingDB component.

Library of the unLHCLoggingDB component used only by unLHCLoggingDB.

PVSS manager usage: CTRL, UI

Constraints:
- PVSS version: 3.11
- Operating System: Linux
*/

#uses "unLHCLoggingDB/unLHCLoggingDBDeprecated.ctl"

/**
* Global mapping to store authentication information
* 
* Structure:
*   g_mAuthN["user"] string username
*   g_mAuthN["pass"] string password
*   g_mAuthN["ok"]   bool authentication ok     
*/
global mapping g_mAuthN;


/**
* Get authentication information, if no information exists then initialise
*
* @return  g_mAuthN  output, authentication information
*/
public mapping unLHCLoggingDBUtils_getAuthentication()
{
    if (!mappingHasKey(g_mAuthN, "user")) {
        unLHCLoggingDBUtils_resetAuthentication();
    }
    return g_mAuthN;
}


/**
* Reset authentication information
*/
public void unLHCLoggingDBUtils_resetAuthentication()
{
    g_mAuthN["user"] = "";
    g_mAuthN["pass"] = "";
    g_mAuthN["ok"] = false;
}


/**
* Open panel so that the user can authenticate
*
* @param user       input, forced username
*/
public bool unLHCLoggingDBUtils_openAuthenticationPanel(string user = "")
{
    dyn_float dfState;
    dyn_string dsUserPass;

    ChildPanelOnCentralModalReturn("objects/unLHCLoggingDB/refUnLHCLoggingDBAuthentication.pnl", 
                                   "Authentication",
                                   makeDynString("$user:" + user),
                                   dfState,
                                   dsUserPass
                                  );

    g_mAuthN["user"] = dsUserPass[1];
    g_mAuthN["pass"] = dsUserPass[2];
    g_mAuthN["ok"] = (dfState[1] == 1);
    return (g_mAuthN["ok"]);
}

/**
* Animate ProgressBar and show some percentage
*
* @param iPercent       input, percent 0 ... 100
*/
public void unLHCLoggingDBUtils_animateProgressBar(int iPercent)
{
    if (isFunctionDefined("shapeExists")) { // if we call the function from a control script shapeExists is not defined
        if (shapeExists("progressBar")) { // only if there is a shape called progressBar
            progressBar.progress(iPercent);
        }
    }
}

/**
* Show hidden column, or hide visible column of a table
*
* @param table       input, name of the table
* @param column      input, name of the column
*/
public void unLHCLoggingDBUtils_toggleColumnVisibility(string table, string column)
{
  shape t = getShape(table);
  if(t) {
    t.namedColumnVisibility(column, !t.namedColumnVisibility(column));
  } 
}


/**
* Make a mapping (missing function in PVSS, to be replaced with standard function, once we have it in a core library)
*
* @param first       input, variable list of key value pairs
* @return            a mapping 
*/
public mapping unLHCloggingDBUtils_makeMapping(anytype first, ...) 
{
  mapping m;
  int len;
  va_list parameters;

  len = va_start(parameters);
  try {
    m[first] = parameters[1];
  }catch {
    DebugTN("No parameters provided past first key: " + first);
    return m;
  }

  for (int i=2; i<=len; i=i+2) {
    try {
      m[parameters[i]] = parameters[i+1];
    } catch {
      DebugTN("Uneven amount of key => value pairs, or key => value pairs dont make sense (provided key was not a string)? Last key handled: " + parameters[i]);
      return m;
    }
  }

  va_end(parameters);
  return m;
}

